require 'benchmark'
require 'gremlin'
require 'securerandom'


include Benchmark

print "Starting benchmarks..."

orig_std_out = STDOUT.clone
STDOUT.reopen(File::open('/dev/null', 'w'))
iterations = ARGV[-1].to_i

counter_init_benchmarks = (1..iterations).map do |n|
    Benchmark.benchmark(CAPTION, 35, FORMAT) do |x|
        counter_name = "test_#{SecureRandom.hex(5)}_counter".to_sym
        x.report("Counter initialize") { Gremlin::Instruments::Counter.new counter_name, "placeholder", {} }
    end
end

counter_init_and_fetch_benchmarks = (1..iterations).map do |n|
    Benchmark.benchmark(CAPTION, 35, FORMAT) do |x|
        counter_name = "test_#{SecureRandom.hex(5)}_counter".to_sym
        x.report("Counter initialize then fetch") do
            Gremlin::Instruments::Counter.new counter_name, "placeholder", {}
            Gremlin.registry.get counter_name
        end
    end
end

counter_init_and_incr_benchmarks = (1..iterations).map do |n|
    Benchmark.benchmark(CAPTION, 35, FORMAT) do |x|
        counter_name = "test_#{SecureRandom.hex(5)}_counter".to_sym
        x.report("Counter increment") do
            counter = Gremlin::Instruments::Counter.new counter_name, "placeholder", {}
            counter.increment
        end
    end
end

gauge_init_benchmarks = (1..iterations).map do |n|
    Benchmark.benchmark(CAPTION, 35, FORMAT) do |x|
        gauge_name = "test_#{SecureRandom.hex(5)}_gauge".to_sym
        gauge_set_val = rand(1..25000000)
        x.report("Gauge initialize") { Gremlin::Instruments::Gauge.new gauge_name, "placeholder", {} }
    end
end

gauge_init_and_fetch_benchmarks = (1..iterations).map do |n|
    Benchmark.benchmark(CAPTION, 35, FORMAT) do |x|
        gauge_name = "test_#{SecureRandom.hex(5)}_gauge".to_sym
        gauge_set_val = rand(1..25000000)
        x.report("Gauge fetch") do
            Gremlin::Instruments::Gauge.new gauge_name, "placeholder", {}
            Gremlin.registry.get gauge_name
        end
    end
end

gauge_init_and_set_benchmarks = (1..iterations).map do |n|
    Benchmark.benchmark(CAPTION, 35, FORMAT) do |x|
        gauge_name = "test_#{SecureRandom.hex(5)}_gauge".to_sym
        gauge_set_val = rand(1..25000000)
        x.report("Gauge set") do
            gauge = Gremlin::Instruments::Gauge.new gauge_name, "placeholder", {}
            gauge.set({ :foo => "baz" }, gauge_set_val)
        end
    end
end

summary_init_benchmarks = (1..iterations).map do |n|
    Benchmark.benchmark(CAPTION, 35, FORMAT) do |x|
        summary_name = "test_#{SecureRandom.hex(5)}_summary".to_sym
        summary_observe_val = rand(1..25000000)
        x.report("Summary initialize") { Gremlin::Instruments::Summary.new summary_name.to_sym, "placeholder", {} }
    end
end

summary_init_and_fetch_benchmarks = (1..iterations).map do |n|
    Benchmark.benchmark(CAPTION, 35, FORMAT) do |x|
        summary_name = "test_#{SecureRandom.hex(5)}_summary".to_sym
        summary_observe_val = rand(1..25000000)
        x.report("Summary fetch") do
            Gremlin::Instruments::Summary.new summary_name.to_sym, "placeholder", {}
            Gremlin.registry.get summary_name
        end
    end
end

summary_init_and_observe_benchmarks = (1..iterations).map do |n|
    Benchmark.benchmark(CAPTION, 35, FORMAT) do |x|
        summary_name = "test_#{SecureRandom.hex(5)}_summary".to_sym
        summary_observe_val = rand(1..25000000)
        x.report("Summary observe") do 
            summary = Gremlin::Instruments::Summary.new summary_name.to_sym, "placeholder", {}
            (1..iterations).each do
              summary.observe({ :bar => "foo" }, summary_observe_val)
            end
        end
    end
end

STDOUT.reopen(orig_std_out)
print "Done!\n"

counter_init_agg           = counter_init_benchmarks.flatten.inject(0) { |sum, x| sum + x.real }
counter_init_and_incr_agg  = counter_init_and_incr_benchmarks.flatten.inject(0) { |sum, x| sum + x.real }
counter_init_and_fetch_agg = counter_init_and_fetch_benchmarks.flatten.inject(0) { |sum, x| sum + x.real }

puts "--------------------------------------------------------"
puts ""
puts "Counter Initialize Total:             #{counter_init_agg*1000} ms"
puts "Counter Initialize Avg:               #{counter_init_agg/iterations*1000} ms over #{iterations} iterations"
puts "Counter Initialize and Fetch Total:   #{counter_init_and_fetch_agg*1000} ms"
puts "Counter Initialize and Fetch Avg:     #{counter_init_and_fetch_agg/iterations*1000} ms over #{iterations} iterations"
puts "Counter Initialize and Fetch Total:   #{counter_init_and_incr_agg*1000} ms"
puts "Counter Initialize and Fetch Avg:     #{counter_init_and_incr_agg/iterations*1000} ms over #{iterations} iterations"
puts ""

gauge_init_agg           = gauge_init_benchmarks.flatten.inject(0) { |sum, x| sum + x.real }
gauge_init_and_fetch_agg = gauge_init_and_fetch_benchmarks.flatten.inject(0) { |sum, x| sum + x.real }
gauge_init_and_set_agg   = gauge_init_and_set_benchmarks.flatten.inject(0) { |sum, x| sum + x.real }

puts "--------------------------------------------------------"
puts ""
puts "Gauge Initialize Total:               #{gauge_init_agg*1000} ms"
puts "Gauge Initialize Avg:                 #{gauge_init_agg/iterations*1000} ms over #{iterations} iterations"
puts "Gauge Initialize and Fetch Total:     #{gauge_init_and_fetch_agg*1000} ms"
puts "Gauge Initialize and Fetch Avg:       #{gauge_init_and_fetch_agg/iterations*1000} ms over #{iterations} iterations"
puts "Gauge Initialize and Set Total:       #{gauge_init_and_set_agg*1000} ms"
puts "Gauge Initialize and Set Avg:         #{gauge_init_and_set_agg/iterations*1000} ms over #{iterations} iterations"
puts ""
puts "--------------------------------------------------------"

summary_init_agg               = summary_init_benchmarks.flatten.inject(0) { |sum, x| sum + x.real }
summary_init_and_fetch_agg     = summary_init_and_fetch_benchmarks.flatten.inject(0) { |sum, x| sum + x.real }
summary_init_and_observe_agg   = summary_init_and_observe_benchmarks.flatten.inject(0) { |sum, x| sum + x.real }

puts ""
puts "Summary Initialize Total:             #{summary_init_agg*1000} ms"
puts "Summary Initialize Avg:               #{summary_init_agg/iterations*1000} ms over #{iterations} iterations"
puts "Summary Initialize and Fetch Total:   #{summary_init_and_fetch_agg*1000} ms"
puts "Summary Initialize and Fetch Avg:     #{summary_init_and_fetch_agg/iterations*1000} ms over #{iterations} iterations"
puts "Summary Initialize and Observe Total: #{summary_init_and_observe_agg*1000} ms"
puts "Summary Initialize and Observe Avg:   #{summary_init_and_observe_agg/(iterations*iterations)*1000} ms over #{iterations} observations with #{iterations} iterations"
puts ""
puts "--------------------------------------------------------"
