# Supporting librarires
require "active_support/cache"
require "active_support/notifications" if defined?(::Rails)
require "active_support/subscriber" if defined?(::Rails)
require "erb"
require "json"
require "quantile"
require "redis"
require "yaml"

# Gem internals
require "gremlin/version"
require "gremlin/registry"
require "gremlin/instruments"
require "gremlin/instruments/counter"
require "gremlin/instruments/gauge"
require "gremlin/instruments/summary"
require "gremlin/quantile"
require "gremlin/notification_observer"

# Rails support
require "gremlin/engine" if defined?(::Rails::Engine)
require "gremlin/railtie" if defined?(::Rails)

module Gremlin
  class << self
    attr_reader :configuration

    def registry
      @registry ||= Gremlin::Registry.new
    end

    def configuration
      unless @configuration
        @configuration = configure do |c|
          c.redis = { :host => "localhost", :port => 6379, :db => 15 }
          c.enabled = true
        end
      end
      @configuration
    end

    def configure
      config = Config.new
      yield config
      config.enabled = true if config.enabled.nil?
      @configuration = config
    end

    def enabled?
      @configuration.enabled
    end

    def disabled?
      not enabled?
    end
  end

  class Config < Struct.new(:redis, :enabled)
  end
end
