module Gremlin
  module Instruments
    class Base
      attr_accessor :name, :base_labels, :docstring

      def initialize(name, docstring="placeholder help string", base_labels={})
        @name = name
        @docstring = docstring
        @base_labels = base_labels

        @r = Redis.new(**Gremlin.configuration.redis)
      end

      def delete
        @r.del retention_key
      end

      def retention_get
        @r.hgetall retention_key
      end

      def parse(value)
        {
          name: @name,
          type: type_string,
          help: help_string,
          values: value.each_with_object({}) { |(l,v), m| m[JSON.parse(l).merge(@base_labels)] = cast(v) }
        }
      end

      def repr
        parse(retention_get)
      end

      def repr_and_delete
        val = nil
        @r.pipelined do
          val = retention_get
          @r.del retention_key
        end
        v = parse(val.value)
        return v
      end

      def node
        `hostname`.strip
      end

      def retention_key
        nil
      end

      def type
        nil
      end

      def type_string
        "# TYPE #{@name} #{type}"
      end

      def help
        @docstring
      end

      def help_string
        "# HELP #{@name} #{help}"
      end

      def cast(v)
        case type
        when :counter
          v.to_i
        when :gauge
          v.to_f
        else
          v
        end
      end

      def values
        retention_get.each_with_object({}) do |(labels, value), memo|
          memo[JSON.parse(labels)] = cast(value)
        end
      end
    end
  end
end
