module Gremlin
  module Quantile
    class Estimator

      attr_accessor :invariants

      def initialize(values: nil, invariants: nil)
        if invariants.nil?
          invariants = [Gremlin::Quantile::Percentile.new(0.5), Gremlin::Quantile::Percentile.new(0.9), Gremlin::Quantile::Percentile.new(0.99)]
        end

        @invariants = invariants
        @values = values || []
      end

      def observe(value)
        @values << value
        sum
      end

      def sum
        @values.reduce { |sum, x| sum += x }
      end

      def values
        @values.sort!
        invariants.inject(Hash.new) do |doc, invariant|
          doc[invariant.percent] = invariant.calculate(@values)
          doc
        end
      end

      def serialize
        JSON.dump({
            invariants: @invariants.map { |i| i.to_h },
            values: @values
        })
      end

      def self.deserialize(message)
        doc = JSON.parse(message || "{}")
        new(values: doc["values"], invariants: doc["invariants"])
      end
    end
  end
end
