module Gremlin
  class Registry
    AlreadyRegisteredError = Class.new StandardError
    NotRegisteredError = Class.new StandardError
    NotAMetricError = Class.new StandardError

    def initialize
      @r = Redis.new(**Gremlin.configuration.redis)
    end

    def node
      `hostname`.strip
    end

    def retention_key
      "gremlin_prometheus_#{node}_metrics_registry"
    end

    def retention_get
      @r.hgetall retention_key
    end

    def metrics
      retention_get.values.map do |m|
        deserialize(m)
      end
    end

    def dump_metrics
      retention_get
    end

    def get(name)
      m = retention_get[name.to_s]
      deserialize(m) unless m.nil?
    end

    def exist?(name)
      @r.hexists retention_key, name.to_s
    end

    def register(instrument)
      name = instrument.name
      set = @r.hset retention_key, name, serialize(instrument)

      instrument
    end

    def deregister(instrument)
      name = instrument.name
      raise NotRegisteredError.new() if @r.hdel(retention_key, name) == 0
      metrics
    end

    def deserialize(metric)
      m = JSON.parse(metric)
      Gremlin::Instruments.const_get(m["type"].capitalize).new(m["name"].to_sym, m["docstring"], m["base_labels"])
    end

    def serialize(metric)
      raise NotAMetricError.new("Specified object is not an instrument and cannot be serialized") unless metric.is_a? Gremlin::Instruments::Base
      {
        type: metric.type,
        name: metric.name.to_s,
        docstring: metric.docstring,
        base_labels: metric.base_labels
      }.to_json
    end
  end
end
